/*
* SimpleFrameDifferencing
* created by Rob Saunders 
* March 2009
*/

import processing.video.*;

Capture cameraFrame;
PImage previousFrame;
PImage currentFrame;
PImage differenceFrame;
PImage displayFrame;

void setup() {
  size(640, 480);
  cameraFrame = new Capture(this, width, height, 30);
  previousFrame = new PImage(width, height);
  currentFrame = new PImage(width, height);
  differenceFrame = new PImage(width, height);
  displayFrame = new PImage(width, height);
}

void draw() {
  // Draw the display frame buffer to the sketch window
  // NOTE: We use the display frame buffer rather than the difference
  // frame buffer to avoid drawing partially computed frames
  image(displayFrame, 0, 0);
}

// This function is called when a new frame is ready from the camera
void captureEvent(Capture cameraFrame) {
  // First we read the frame from the camera
  cameraFrame.read();
  // Next we store the current frame buffer into the previous frame buffer
  previousFrame.copy(currentFrame, 0, 0, width, height, 0, 0, width, height);
  // Now we store the frame from the camera into the current frame buffer
  currentFrame.copy(cameraFrame, 0, 0, width, height, 0, 0, width, height);
  // Copy the previous frame to the difference frame
  differenceFrame.copy(previousFrame, 0, 0, width, height, 0, 0, width, height);
  // Blend the current frame with the previous frame using the DIFFERENCE function
  differenceFrame.blend(currentFrame, 0, 0, width, height, 0, 0, width, height, DIFFERENCE);
  
  // ***** the line below shows an example for VARIATION ***** 
  // finding, studying and adding a function like this would result in a DISTINCTION (75) 
  // for your submission because (1) it adds to what this sketch can do -it’s not 
  // an arbitrary extension, and (2) we haven’t covered it in class yet
  // This threshold filter will show every different pixel (from prev. frame) in white
  differenceFrame.filter(THRESHOLD, 0.20);
  //***** end of Variation(****

  // Copy the difference frame to the display frame
  displayFrame.copy(differenceFrame, 0, 0, width, height, 0, 0, width, height);
}
